<?php
/*

This code is a PHP script that can be used as a chatbot transfer target. It asks the user a series 
of pre-configured questions and then transfers the chat to a human agent for further assistance.

Here's a breakdown of the code:

Configuration:

$DEBUG: Controls the level of debug logging (0 for no logging, 1 for basic logging).

$config: Defines various messages and settings for the chatbot, including:
    last_action:        The action to take after completing all questions (default: transfer).
    invalid_answer_message: Message to display when the user provides an invalid answer.
    close_message:          Message to display when the chatbot closes the conversation.
    transfer_message:       Message to display when the chatbot transfers the chat to a human agent.

$questions: Array of questions to ask the user, each with the following properties:
    body:          The text of the question.
    response_type: The type of answer expected (text, range, or menu).
    options:       Additional information depending on the response type (e.g., valid range for range questions, options for menu questions).
    next_action:   The next step after the user answers the question (e.g., index of the next question, special action like transfer).

Processing Flow:

Retrieve Input:

The script reads the JSON data sent from the chatbot platform.

It retrieves the state field, which indicates the current stage of the conversation.

Validate Answer (if applicable):

If the state indicates a previous question, the answer is validated based on the question type and expected response.
If the answer is invalid, an error message is displayed.
If the answer is valid, the next step is determined.

Retrieve Next Question:
Based on the current state or validated answer, the next question is retrieved from the $questions array.

Respond to User:
The script sends a response to the chatbot platform containing the next question or a message based on the current stage (e.g., close message, transfer message).

Update State:
The state is updated based on the current stage and next step.

Additional Notes:

The script supports different types of questions:
Text: Open-ended questions where the user can type any response.
Range: Questions where the user selects a number from a defined range.
Menu: Questions where the user chooses an option from a list.
The script can be used to collect information from the user before transferring the chat to a human agent.
This is just a basic example, and the script can be customized further to meet specific needs.

*/

$DEBUG=1;

if(!is_callable('array_key_first')) {
   function array_key_first($myarray) {
       reset($myarray);
       return key($myarray);
   }
}

$config = array(
   'last_action'        => 'transfer',
   'invalid_answer_message' => 'No entendí la respuesta.',
   'close_message'          => 'Gracias!',
   'transfer_message'       => 'Gracias, lo contactaré con un asistente humano'
);

// override all hardcoded options if we have a bot name and ini file
if(isset($_REQUEST['bot'])) {
    $filename = getcwd()."/".$_REQUEST['bot'].".ini";
    if(is_readable($filename)) {

        $ini = parse_ini_file($filename,false);
        $questions = json_decode(base64_decode($ini['steps']),true);

        if(isset($ini['last_action'])) {
            if($ini['last_action']!='') {
                $config['last_action']=$ini['last_action'];
            }
        }

        if(isset($ini['invalid_answer_message'])) {
            if($ini['invalid_answer_message']!='') {
                $config['invalid_answer_message']=$ini['invalid_answer_message'];
            }
        }

        if(isset($ini['close_message'])) {
            if($ini['close_message']!='') {
                $config['close_message']=$ini['close_message'];
            }
        }

        if(isset($ini['transfer_message'])) {
            if($ini['transfer_message']!='') {
                $config['transfer_message']=$ini['transfer_message'];
            }
        }
    }
}

/*

$questions = array(
    array(
        "body"          => "Cuál es su nombre?",
        "response_type" => "text",
        "next_action"   => "next"
    ),
    array(
        "body"          => "Del 1 al 5, cómo nos calificaría, siendo 1 muy malo y 5 muy bueno?",
        "response_type" => "range",
        "options"       => array(1,2,3,4,5),
        "next_action"   => "next"
    ),
    array(
        "body"          => "Qué día está libre?",
        "response_type" => "text",
        "next_action"   => "next"
    ),
    array(
        "body"          => "Por favor elija una opción:\n1) Consulta General\n2) Retiro de puntos\n3) Vacuna o Inyectable",
        "response_type" => "menu",
        "options"       => array("1" => 4, "2" => 5, "3" => 6)
    ),
    array(
        "body"          => "Muy bien, ahora elija la especialidad:\nA) Consulta general\nB) Consulta Ortopédica\nC) Consulta de control",
        "response_type" => "menu",
        "options"       => array("A" => 7, "B" => 7, "C" => 7)
    ),
    array(
        "body"          => "Muy bien, que tipo?:\nA) Retiro de puntos de Cirugía Ortopédica\nB) Retiro de Puntos de otra cirugía",
        "response_type" => "menu",
        "options"       => array("A" => 7, "B" => 7)
    ),
    array(
        "body"          => "Muy bien, que tipo de vacuna?:\nA) Vacunas anuales\nB) Vacunas de cachorros\nC) Inyectable de control",
        "response_type" => "menu",
        "options"       => array("A" => 7, "B" => 7, "C" => 7)
    ),
    array(
        "body"          => "Gracias! Lo contactaremos con un asesor de soporte técnico",
        "response_type" => "transfer",
        "options"       => "Soporte",
        "next_action"   => "next"
    )
);

*/

$json = file_get_contents('php://input');
$data = json_decode($json,1);

if(!isset($data)) { exit; }

log_debug("interactive bot start");

$validate_command = '';

if($data['state']=='') {
    // First interation with bot, retrieve first question (index 0 of questions array)
    $step=0;
    log_debug("first interaction as we have no state");

} else {

    log_debug("its not the first interaction");
    // Not first interaction, validate answer from previous step
    // the state field will hold the string "previous_step-step"
    // to know where we came from (to validate the answer from a previous question), 
    // and when we are to print the next message/question after validating

    list($prev_step,$step) = preg_split("/-/",substr($data['state'],1,-1)); // state is enclosed in double quotes, strip them

    $step      = intval($step);
    $prev_step = intval($prev_step);
    log_debug(print_r($data,1));
    $answer    = $data['body'];
    log_debug("current step: $step, previous step: $prev_step (".$data['state'].")");

    $validate_command = validate_answer($prev_step,$answer);

    log_debug("validate answer ( $prev_step, $answer) returned comand: $validate_command");
}

if($validate_command=='[wrong]') {
    log_debug("wrong");
    list($msg,$next) = retrieve_question($prev_step);
    $return = array("body"=>array($config['invalid_answer_message'],$msg),"attach"=>'',"state"=>substr($data['state'],1,-1));
    echo json_encode($return);
    die();
} else if($validate_command=='[close]') {
    $return = array("body"=>$config['close_message']." [close]","attach"=>'',"state"=>'');
    echo json_encode($return);
    die();
} else if($validate_command=='[transfer]') {
    $return = array("body"=>$config['transfer_message']." [transfer]","attach"=>'',"state"=>'');
    echo json_encode($return);
    die();
} else if(preg_match("/\[next=(\d+)\]/", $validate_command, $matches)) {
    $step = $matches[1];
    log_debug("next step from menu ".$step);
}

// check route step based on user message
$step =  follow_route($step,$data['body']);

list($msg,$state) = retrieve_question($step);

log_debug("retrieve question $step returned  $msg, set state to $state, current answer ($answer)");

// Append configuration close or transfer messages if the command was received
if($msg=='[close]')    $msg = $config['close_message']." [close]";
if($msg=='[transfer]') $msg = $config['transfer_message']." [transfer]";

// Responds to chatbot with actual message and exists
$return = array("body"=>$msg,"attach"=>'',"state"=>$state);
echo json_encode($return);
exit;


function follow_route($step,$answer) {
    global $questions, $config, $data;

    if(!isset($questions[$step])) return array("[".$config['last_action']."]","end");
    if($questions[$step]['response_type']=='route') {
        $valid_answers = array();
        $step = $questions[$step]['nomatch'];
        foreach($questions[$step]['options'] as $key=>$val) {
            $valid_answers[strtolower($key)]=$val;
        }
        $found=0;
        foreach($valid_answers as $key=>$goto) {
            if(preg_match("/$key/i",$answer)) {
                 $found=1;
                 $step = $goto;
                 break;
            }
        }
    }
    return $step;
}

function retrieve_question($step) {
    global $questions, $config, $data;
    if(!isset($questions[$step])) return array("[".$config['last_action']."]","end");
    $question_body = $questions[$step]['body'];
    if($questions[$step]['response_type']=='transfer') {
       if($questions[$step]['options']!='') {
           $question_body.="[transfer^".$questions[$step]['options']."]";
       } else {
           $question_body.="[transfer]";
       }
    } else if($questions[$step]['response_type']=='close') {
        $question_body.="[close]";
    } else if($questions[$step]['response_type']=='menu') {

        if(strlen(array_key_first($questions[$step]['options']))>1 && count($questions[$step]['options'])<4 && strpos($data['chatid'], "meta_") === 0) {
            // convert menu to whatsapp interactive buttons
            // menu option is not a single character/digit, asume interactive message (up to three buttons only)

            $buttons = array();
            $i=0;
            foreach($questions[$step]['options'] as $key=>$goto) {
                if($i>2) break;
                $buttons[] = array("type"=>"reply","reply"=>array("id"=>"id".$i,"title"=>$key));
                $i++;
            }

            if($i>0) {
                $body = array(
                    "type"   =>"button",
                    "body"   => array( "text" => $questions[$step]['body'] ),
                    "action" => array(
                      "buttons" => $buttons
                    )
                );
                $unicodeChar = '\u0008';
                $null = json_decode('"'.$unicodeChar.'"');
                $question_body=$null.json_encode($body);
            }
        } else if(strlen(array_key_first($questions[$step]['options']))>1 && (strpos($data['chatid'], "telegram_") === 0 || strpos($data['chatid'], "webchat_") === 0 || strpos($data['chatid'], "wuzapi_") === 0)) {
            $unicodeChar = '\u0008';
            $null = json_decode('"'.$unicodeChar.'"');
            $buttons = array();
            $i=0;
            foreach($questions[$step]['options'] as $key=>$goto) {
                if($i>2) break;
                $buttons[] = array("type"=>"reply","reply"=>array("id"=>"id".$i,"title"=>$key));
                $i++;
            }

            $body = array('body'=>array("text"=>$questions[$step]['body']),'action'=>array('buttons'=>$buttons),"type"=>"button");
            $question_body=$null.json_encode($body);
        }

        $questions[$step]['next_action']='same';
    }

    // Define new state for returning 
    $next = $questions[$step]['next_action'];
    if($next=='next') {
        $next_step = $step+1;
    } else if($next=='same') {
        $next_step = $step;
    } else {
        $next_step = intval($next);
    }
    $state = "$step-$next_step";

    return array($question_body,$state);
}

function validate_answer($n,$answer) {
    global $questions, $config;
    if(!isset($questions[$n])) {
        return "[".$config['last_action']."]";
    }
    if($questions[$n]['response_type']=='text') {
        if($questions[$n]['next_action']!='next') { 
            $next_step = $questions[$n]['next_action'];
            return "[next=$next_step]";
        }
    } else 
    if($questions[$n]['response_type']=='range') {
        $valid_answers = $questions[$n]['options'];
        if(!in_array($answer,$valid_answers)) {
            return "[wrong]";
        } else {
            if($questions[$n]['next_action']!='next') { 
                $next_step = $questions[$n]['next_action'];
                return "[next=$next_step]";
            }
        }
    } else
    if($questions[$n]['response_type']=='menu') {
        log_debug("validate menu, answer given: $answer");
        $valid_answers=array();
        foreach($questions[$n]['options'] as $key=>$val) {
            $valid_answers[strtolower($key)]=$val;
        }
        $found=0;
        foreach($valid_answers as $key=>$goto) {
            if(preg_match("/$key/i",$answer)) {
                 $found=1;
                 $next_step = $goto;
                 break;
            }
        }
        if($found==0) {
            return "[wrong]";
        }
        return "[next=$next_step]";
    }
    return '';
}

function log_debug($text) {
    global $DEBUG;
    $logfile = basename(__FILE__,'.php');
    if($DEBUG==1) {
        file_put_contents("/tmp/${logfile}.log",$text."\n",FILE_APPEND);
    }
}
